@@ -1,4 +1,9 @@
 <?php
+/*
+ * Copyright (c) 2020. The Nextcloud Bookmarks contributors.
+ *
+ * This file is licensed under the Affero General Public License version 3 or later. See the COPYING file.
+ */
 
 namespace OCA\Bookmarks\Db;
 
@@ -17,6 +22,8 @@ use OCP\EventDispatcher\IEventDispatcher;
 use OCP\ICache;
 use OCP\IConfig;
 use OCP\IDBConnection;
+use PDO;
+use function call_user_func;
 
 /**
  * Class TreeMapper
@@ -84,6 +91,18 @@ class TreeMapper extends QBMapper {
 	 * @var IQueryBuilder
 	 */
 	private $insertQuery;
+	/**
+	 * @var IQueryBuilder
+	 */
+	private $parentQuery;
+	/**
+	 * @var array
+	 */
+	private $getChildrenQuery;
+	/**
+	 * @var IQueryBuilder
+	 */
+	private $getChildrenOrderQuery;
 
 	/**
 	 * FolderMapper constructor.
@@ -116,6 +135,13 @@ class TreeMapper extends QBMapper {
 		$this->publicFolderMapper = $publicFolderMapper;
 
 		$this->insertQuery = $this->getInsertQuery();
+		$this->parentQuery = $this->getParentQuery();
+		$this->getChildrenOrderQuery = $this->getGetChildrenOrderQuery();
+		$this->getChildrenQuery = [
+			self::TYPE_BOOKMARK => $this->getFindChildrenQuery(self::TYPE_BOOKMARK),
+			self::TYPE_FOLDER => $this->getFindChildrenQuery(self::TYPE_FOLDER),
+			self::TYPE_SHARE => $this->getFindChildrenQuery(self::TYPE_SHARE)
+		];
 	}
 
 	/**
@@ -127,7 +153,7 @@ class TreeMapper extends QBMapper {
 	 * @return Entity the entity
 	 */
 	protected function mapRowToEntityWithClass(array $row, string $entityClass): Entity {
-		return \call_user_func($entityClass . '::fromRow', $row);
+		return call_user_func($entityClass . '::fromRow', $row);
 	}
 
 
@@ -168,14 +194,17 @@ class TreeMapper extends QBMapper {
 
 	/**
 	 * @param string $type
+	 * @param array $cols
+	 * @param IQueryBuilder|null $queryBuilder
 	 * @return IQueryBuilder
 	 */
-	protected function selectFromType(string $type): IQueryBuilder {
-		$qb = $this->db->getQueryBuilder();
+	protected function selectFromType(string $type, array $cols = [], IQueryBuilder $queryBuilder = null): IQueryBuilder {
+		$qb = $queryBuilder ?? $this->db->getQueryBuilder();
+		$qb->resetQueryPart('from');
 		$qb
-			->select(array_map(static function ($col) {
+			->select(array_merge(array_map(static function ($col) {
 				return 'i.' . $col;
-			}, $this->entityColumns[$type]))
+			}, $this->entityColumns[$type]), $cols))
 			->from($this->entityTables[$type], 'i');
 		return $qb;
 	}
@@ -193,18 +222,43 @@ class TreeMapper extends QBMapper {
 		return $qb;
 	}
 
+	protected function getParentQuery(): IQueryBuilder {
+		$qb = $this->selectFromType(self::TYPE_FOLDER);
+		$qb
+			->join('i', 'bookmarks_tree', 't', $qb->expr()->eq('t.parent_folder', 'i.id'))
+			->where($qb->expr()->eq('t.id', $qb->createParameter('id')))
+			->andWhere($qb->expr()->eq('t.type', $qb->createParameter('type')));
+		return $qb;
+	}
+
+	protected function getGetChildrenOrderQuery(): IQueryBuilder {
+		$qb = $this->db->getQueryBuilder();
+		$qb
+			->select('id', 'type', 'index')
+			->from('bookmarks_tree')
+			->where($qb->expr()->eq('parent_folder', $qb->createParameter('parent_folder')))
+			->orderBy('index', 'ASC');
+		return $qb;
+	}
+
+	protected function getFindChildrenQuery(string $type): IQueryBuilder {
+		$qb = $this->selectFromType($type);
+		$qb
+				->join('i', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 'i.id'))
+				->where($qb->expr()->eq('t.parent_folder', $qb->createParameter('parent_folder')))
+				->andWhere($qb->expr()->eq('t.type', $qb->createNamedParameter($type)))
+				->orderBy('t.index', 'ASC');
+		return $qb;
+	}
+
 	/**
 	 * @param int $folderId
 	 * @param string $type
 	 * @return array|Entity[]
 	 */
 	public function findChildren(string $type, int $folderId): array {
-		$qb = $this->selectFromType($type);
-		$qb
-			->join('i', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 'i.id'))
-			->where($qb->expr()->eq('t.parent_folder', $qb->createPositionalParameter($folderId)))
-			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter($type)))
-			->orderBy('t.index', 'ASC');
+		$qb = $this->getChildrenQuery[$type];
+		$qb->setParameter('parent_folder', $folderId);
 		return $this->findEntitiesWithType($qb, $type);
 	}
 
@@ -216,11 +270,11 @@ class TreeMapper extends QBMapper {
 	 * @throws MultipleObjectsReturnedException
 	 */
 	public function findParentOf(string $type, int $itemId): Entity {
-		$qb = $this->selectFromType(self::TYPE_FOLDER);
-		$qb
-			->join('i', 'bookmarks_tree', 't', $qb->expr()->eq('t.parent_folder', 'i.id'))
-			->where($qb->expr()->eq('t.id', $qb->createPositionalParameter($itemId)))
-			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter($type)));
+		$qb = $this->parentQuery;
+		$qb->setParameters([
+			'id' => $itemId,
+			'type' => $type,
+		]);
 		return $this->findEntityWithType($qb, $type);
 	}
 
@@ -230,11 +284,11 @@ class TreeMapper extends QBMapper {
 	 * @return array|Entity[]
 	 */
 	public function findParentsOf(string $type, int $itemId): array {
-		$qb = $this->selectFromType(self::TYPE_FOLDER);
-		$qb
-			->join('i', 'bookmarks_tree', 't', $qb->expr()->eq('t.parent_folder', 'i.id'))
-			->where($qb->expr()->eq('t.id', $qb->createPositionalParameter($itemId)))
-			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter($type)));
+		$qb = $this->parentQuery;
+		$qb->setParameters([
+			'id' => $itemId,
+			'type' => $type,
+		]);
 		return $this->findEntitiesWithType($qb, self::TYPE_FOLDER);
 	}
 
@@ -330,8 +384,9 @@ class TreeMapper extends QBMapper {
 				->where($qb->expr()->isNull('t.id'));
 			$orphanedBookmarks = $qb->execute();
 			while ($bookmark = $orphanedBookmarks->fetchColumn()) {
-				$bm = $this->bookmarkMapper->find($bookmark);
-				$this->bookmarkMapper->delete($bm);
+				$qb = $this->db->getQueryBuilder();
+				$qb->delete('bookmarks', 'b')
+					->where($qb->expr()->eq('b.id', $qb->createPositionalParameter($bookmark)));
 			}
 
 			return;
@@ -392,20 +447,12 @@ class TreeMapper extends QBMapper {
 			throw new UnsupportedOperation('Cannot move Bookmark');
 		}
 		try {
+			// Try to find current parent
 			/** @var Folder $currentParent */
 			$currentParent = $this->findParentOf($type, $itemId);
 
-			$ancestor = new Folder();
-			$ancestor->setId($newParentFolderId);
-			while (true) {
-				if ($ancestor->getId() === $itemId) {
-					throw new UnsupportedOperation('Cannot nest a folder inside one of its descendants');
-				}
-				try {
-					$ancestor = $this->findParentOf(self::TYPE_FOLDER, $ancestor->getId());
-				} catch (DoesNotExistException $e) {
-					break;
-				}
+			if ($this->hasDescendant($itemId, self::TYPE_FOLDER, $newParentFolderId)) {
+				throw new UnsupportedOperation('Cannot nest a folder inside one of its descendants');
 			}
 
 			// Item currently has a parent => move.
@@ -600,23 +647,14 @@ class TreeMapper extends QBMapper {
 	 * @param int $layers The amount of levels to return
 	 * @return array the children each in the format ["id" => int, "type" => 'bookmark' | 'folder' ]
 	 */
-	public function getChildrenOrder($folderId, $layers = 0): array {
-		$qb = $this->db->getQueryBuilder();
-		$qb
-			->select('id', 'type', 'index')
-			->from('bookmarks_tree')
-			->where($qb->expr()->eq('parent_folder', $qb->createPositionalParameter($folderId)))
-			->orderBy('index', 'ASC');
+	public function getChildrenOrder(int $folderId, $layers = 0): array {
+		$qb = $this->getChildrenOrderQuery;
+		$qb->setParameter('parent_folder', $folderId);
 		$children = $qb->execute()->fetchAll();
 
-		$qb = $this->db->getQueryBuilder();
-		$qb
-			->select('folder_id', 'index')
-			->from('bookmarks_shared_folders', 'sf')
-			->innerJoin('sf', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 'sf.id'))
-			->where($qb->expr()->eq('t.parent_folder', $qb->createPositionalParameter($folderId)))
-			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter(self::TYPE_SHARE)))
-			->orderBy('t.index', 'ASC');
+		$qb = $this->getChildrenQuery[self::TYPE_SHARE];
+		$this->selectFromType(self::TYPE_SHARE, ['t.index'], $qb);
+		$qb->setParameter('parent_folder', $folderId);
 		$childShares = $qb->execute()->fetchAll();
 
 		$children = array_map(function ($child) use ($layers, $childShares) {
@@ -680,7 +718,7 @@ class TreeMapper extends QBMapper {
 			->select($qb->func()->count('index', 'count'))
 			->from('bookmarks_tree')
 			->where($qb->expr()->eq('parent_folder', $qb->createPositionalParameter($folderId)));
-		return $qb->execute()->fetch(\PDO::FETCH_COLUMN);
+		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
 	}
 
 	/**
@@ -696,7 +734,7 @@ class TreeMapper extends QBMapper {
 			->innerJoin('b', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 'b.id'))
 			->where($qb->expr()->eq('t.parent_folder', $qb->createPositionalParameter($folderId)))
 			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter(self::TYPE_BOOKMARK)));
-		$countChildren = $qb->execute()->fetch(\PDO::FETCH_COLUMN);
+		$countChildren = $qb->execute()->fetch(PDO::FETCH_COLUMN);
 
 		$qb = $this->db->getQueryBuilder();
 		$qb
@@ -705,7 +743,7 @@ class TreeMapper extends QBMapper {
 			->innerJoin('f', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 'f.id'))
 			->where($qb->expr()->eq('t.parent_folder', $qb->createPositionalParameter($folderId)))
 			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter(self::TYPE_FOLDER)));
-		$childFolders = $qb->execute()->fetchAll(\PDO::FETCH_COLUMN);
+		$childFolders = $qb->execute()->fetchAll(PDO::FETCH_COLUMN);
 
 		foreach ($childFolders as $subFolderId) {
 			$countChildren += $this->countBookmarksInFolder($subFolderId);
@@ -714,37 +752,19 @@ class TreeMapper extends QBMapper {
 	}
 
 	public function getChildren(int $folderId, int $layers = 0) {
-		$qb = $this->db->getQueryBuilder();
-		$cols = array_merge(['index', 't.type'], array_map(static function ($col) {
-			return 'b.' . $col;
-		}, Bookmark::$columns));
-		$qb
-			->select($cols)
-			->from('bookmarks', 'b')
-			->innerJoin('b', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 'b.id'))
-			->where($qb->expr()->eq('t.parent_folder', $qb->createPositionalParameter($folderId)))
-			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter(self::TYPE_BOOKMARK)))
-			->orderBy('t.index', 'ASC');
+		$qb = $this->getChildrenQuery[self::TYPE_BOOKMARK];
+		$this->selectFromType(self::TYPE_BOOKMARK, ['t.index', 't.type'], $qb);
+		$qb->setParameter('parent_folder', $folderId);
 		$childBookmarks = $qb->execute()->fetchAll();
 
-		$qb = $this->db->getQueryBuilder();
-		$qb
-			->select('f.id', 'title', 'user_id', 'index', 't.type')
-			->from('bookmarks_folders', 'f')
-			->innerJoin('f', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 'f.id'))
-			->where($qb->expr()->eq('t.parent_folder', $qb->createPositionalParameter($folderId)))
-			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter(self::TYPE_FOLDER)))
-			->orderBy('t.index', 'ASC');
+		$qb = $this->getChildrenQuery[self::TYPE_FOLDER];
+		$this->selectFromType(self::TYPE_FOLDER, ['t.index', 't.type'], $qb);
+		$qb->setParameter('parent_folder', $folderId);
 		$childFolders = $qb->execute()->fetchAll();
 
-		$qb = $this->db->getQueryBuilder();
-		$qb
-			->select('folder_id', 's.title', 'user_id', 'index', 't.type')
-			->from('bookmarks_shared_folders', 's')
-			->innerJoin('s', 'bookmarks_tree', 't', $qb->expr()->eq('t.id', 's.id'))
-			->where($qb->expr()->eq('t.parent_folder', $qb->createPositionalParameter($folderId)))
-			->andWhere($qb->expr()->eq('t.type', $qb->createPositionalParameter(self::TYPE_SHARE)))
-			->orderBy('t.index', 'ASC');
+		$qb = $this->getChildrenQuery[self::TYPE_SHARE];
+		$this->selectFromType(self::TYPE_SHARE, ['t.index', 't.type'], $qb);
+		$qb->setParameter('parent_folder', $folderId);
 		$childShares = $qb->execute()->fetchAll();
 
 		$children = array_merge($childBookmarks, $childFolders, $childShares);
